/*
 * This file is part of Locana.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.locana.distributor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

public class MeasurementPointsDistributor {

	public static void main(String[] args) {
		int numberOfPoints = 60;
		int minX = 0;
		int maxX = 3300;
		int minY = 0;
		int maxY = 1300;
		int startX = 4329;
		int startY = 3690;
		
		ArrayList<AreaToLeaveOut> areaList = new ArrayList<AreaToLeaveOut>();
		ArrayList<MeasurementPoint> pointList = new ArrayList<MeasurementPoint>();
		
		Random randomizer = new Random();

		// definitions of the areas that should be left out
		// parameters: left, right, bottom, top
		areaList.add(new AreaToLeaveOut(   0,  620,  482,  608)); // Säule 1
		areaList.add(new AreaToLeaveOut( 260, 3300,    0,  340)); // Büros unten
		areaList.add(new AreaToLeaveOut( 260, 3300,  810, 1400)); // Büros oben
		areaList.add(new AreaToLeaveOut( 260,  834,  358,  700)); // Kopierraum
		areaList.add(new AreaToLeaveOut( 834,  908,  492,  618)); // Säule 2
		areaList.add(new AreaToLeaveOut(1330, 2032,  358,  700)); // Teeküche
		areaList.add(new AreaToLeaveOut(2462, 2590,  488,  616)); // Säule 3
		areaList.add(new AreaToLeaveOut(2654, 3150,  358,  700)); // Archiv
		areaList.add(new AreaToLeaveOut(3274, 3300,  492,  620)); // Säule 4
		
		// create the point list
		int pointsToCalculate = 2000 * numberOfPoints;
		int last = 0;
		while (pointList.size() < pointsToCalculate) {
			// find a new point
			int x = randomizer.nextInt(maxX - minX) + minX;
			int y = randomizer.nextInt(maxY - minY) + minY;
			// check if the point collides with one of the areas that should be left out
			boolean collision = false;
			for (int i = 0; i < areaList.size(); ++i) {
				if (areaList.get(i).collides(x, y)) {
					collision = true;
					break;
				}
			}
			if (!collision) { // no collision with one of the areas
				// check if the point already exists in the list
				boolean known = false;
				for (int i = 0; i < pointList.size(); ++i) {
					if ((pointList.get(i).getX() == x) && (pointList.get(i).getY() == y))
						known = true;
				}
				if (!known) {
					pointList.add(new MeasurementPoint(x, y));
				}
			}
			int current = pointList.size() * 100 / pointsToCalculate;
			if (current > last) {
				last = current;
				System.out.println(current + " %");
			}
		}
		
		// sort and output the points
		Collections.sort(pointList);
		int count = 1;
		int degrees;
		for (int i = 0; i < pointList.size(); i += 2000) {
			int x = pointList.get(i).getX();
			int y = pointList.get(i).getY();
			degrees = randomizer.nextInt(360);
			System.out.println(count + ": " + (x / 100.0) + " / " + (y / 100.0) + " (" + ((x / 2) + startX) + " / " + ((y / 2) + startY) + "), " + degrees + " degrees");
			count++;
		}
	}
}
