/*
 * This file is part of Locana.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.locana.misc;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

public class AccessPointLister {
	private static Hashtable<String, Integer> apList = new Hashtable<String, Integer>();
	
	public static void main(String[] args) {
		if (args.length != 1) {
			usage();
			System.exit(1);
		}
		// traceFile öffnen
	    try {
		    System.out.println("Reading the tracefile ...");
	    	BufferedReader in = new BufferedReader(new FileReader(args[0]));
			
		    // zeilenweise einlesen
			int pollCount = 0;
			String line;
			while ((line = in.readLine()) != null) {
				if (line.startsWith("#")) {
					continue;
				}
				// Zeile zerlegen
				String[] components = line.split(";");
				for (String s: components) {
					String[] kv = s.split("=", 2);
					if (kv.length != 2)
						throw new IOException("Expected key=value");
					String key = kv[0];
				    // unwichtige Teile verwerfen
					if ((key.equals("t")) || (key.equals("id")) || (key.equals("pos")) || (key.equals("degree")))
						continue;
					// MAC-Adresse noch unbekannt?
					if (!apList.containsKey(key)) {
						apList.put(key, 1);
					} else {
						apList.put(key, new Integer(apList.get(key) + 1));
					}
				}
				pollCount++;
			}
			System.out.println(pollCount + " lines read");
			// traceFile schliessen
			in.close();
			
			System.out.println("\n" + apList.size() + " access points were found in the tracefile:");
			Vector<String> v = new Vector<String>();
			v.addAll(apList.keySet());
			Collections.sort(v);
			Iterator<String> apIterator = v.iterator();
			while (apIterator.hasNext()) {
				String macAddress = apIterator.next();
				int frequency = apList.get(macAddress);
				System.out.println(macAddress + " (" + frequency + " reading" + (frequency != 1 ? "s" : "") + ")");
			}
			System.out.println();
	    } catch (FileNotFoundException e) {
	    	System.err.println("Couldn't open tracefile\n" + e.toString());
	    	System.out.println();
	    	usage();
		} catch (IOException e) {
			System.err.println("Couldn't read from tracefile\n" + e.toString());
	    	System.out.println();
	    	usage();
		} catch (Exception e) {
	    	System.err.println("Unknown error\n");
	    	e.printStackTrace();
	    	System.out.println();
	    	usage();
	    }
	}
	
	static void usage() {
		System.out.println("usage: java AccessPointLister tracefile.trace");
		System.out.println();
		System.out.println("This program collects details about access points in a given trace file:");
		System.out.println(" - prints the number of access points");
		System.out.println(" - prints the number of readings for each particular access point");
		System.out.println();
	}
}