/*
 * This file is part of Locana.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.locana.misc;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;

public class ExamineTrace {
	private static ArrayList<AccessPoint> apList;
	
	public static void main(String[] args) {
		Hashtable<String, AccessPoint> aps = new Hashtable<String, AccessPoint>();
		Hashtable<String, Boolean> apSeenInThisPoll = new Hashtable<String, Boolean>();
		Hashtable<String, Boolean> apSeenInThisTrace = new Hashtable<String, Boolean>();
		ArrayList<String> minimumMacSetPolls = new ArrayList<String>();
		ArrayList<String> minimumMacSetTraces = new ArrayList<String>();
		Hashtable<String, Boolean> tracefilePositions = new Hashtable<String, Boolean>();
		
		boolean useAllTraces = false;
		
		double xrangeStart = 0.0;
		double yrangeStart = 0.0;
		double xrangeEnd = 2.0;
		double yrangeEnd = 4.0;
		double xstep = 0.5;
		double ystep = 0.5;
		double degstep = 45.0;
		double degmap = 45.0;
		double currentX = -1.0;
		double currentY = -1.0;
		
	    int pollsWithMultipleEntriesPerAP = 0;
	    		
	    if (args.length < 1) {
	        System.out.println();
	        System.out.println("Usage: ExamineTrace traceFile [OPTIONS]");
	        System.out.println("Options:");
	        System.out.println("  -t         index       index of the trace dataset in the file or 'all' to combine all the traces");
	        System.out.println("  -xrange    start end   x-coordinate range of the training set (default: " + xrangeStart + " - " + xrangeEnd + ")");
	        System.out.println("  -yrange    start end   y-coordinate range of the training set (default: " + yrangeStart + " - " + yrangeEnd + ")");
	        System.out.println("  -xstep     stepsize    difference between the x-coordinates of two adjacent positions (default: " + xstep + ")");
	        System.out.println("  -ystep     stepsize    difference between the y-coordinates of two adjacent positions (default: " + ystep + ")");
	        System.out.println("  -degstep   stepsize    difference between the orientation of two successive traces (default: " + degstep + ")");
	        System.out.println("  -degmap    degrees     map the orientation read from the tracefile to a multiple of 'degrees' (default: " + degmap + ")");
	        System.out.println("  -check                 check training set completeness and for unexpected positions and/or orientations");
	        System.out.println();
	        return;
	    }
	    
	    int traceIndex = 0;
	    boolean checkTrainingSet = false;
	    if (args.length > 1) { // es wurden Optionen angegeben
	    	for (int i = 0; i < args.length; i++) {
	    		if (args[i].equalsIgnoreCase("-t")) {
	    			if ((i + 1) < args.length) { // es gibt noch ein weiteres Argument
	    				if (args[i + 1].equals("all")) {
	    					useAllTraces = true;
	    				} else {
	    					traceIndex = Integer.parseInt(args[i + 1]);
	    				}
	    			}
	    		} else if (args[i].equalsIgnoreCase("-xrange")) {
	    			if ((i + 2) < args.length) { // es gibt noch 2 weitere Argumente
	    				xrangeStart = Double.parseDouble(args[i + 1]);
	    				xrangeEnd = Double.parseDouble(args[i + 2]);
	    			}
	    		} else if (args[i].equalsIgnoreCase("-yrange")) {
	    			if ((i + 2) < args.length) { // es gibt noch 2 weitere Argumente
	    				yrangeStart = Double.parseDouble(args[i + 1]);
	    				yrangeEnd = Double.parseDouble(args[i + 2]);
	    			}
	    		} else if (args[i].equalsIgnoreCase("-xstep")) {
	    			if ((i + 1) < args.length) { // es gibt noch ein weiteres Argument
	    				xstep = Double.parseDouble(args[i + 1]);
	    			}
	    		} else if (args[i].equalsIgnoreCase("-ystep")) {
	    			if ((i + 1) < args.length) { // es gibt noch ein weiteres Argument
	    				ystep = Double.parseDouble(args[i + 1]);
	    			}
	    		} else if (args[i].equalsIgnoreCase("-degstep")) {
	    			if ((i + 1) < args.length) { // es gibt noch ein weiteres Argument
	    				degstep = Double.parseDouble(args[i + 1]);
	    			}
	    		} else if (args[i].equalsIgnoreCase("-degmap")) {
	    			if ((i + 1) < args.length) { // es gibt noch ein weiteres Argument
	    				degmap = Double.parseDouble(args[i + 1]);
	    			}
	    		} else if (args[i].equalsIgnoreCase("-check")) {
	    			checkTrainingSet = true;
	    		}
	    	}
	    }
	    // Datenstruktur zur Vollständigkeitskontrolle des Trainingssets erstellen
	    if (checkTrainingSet) {
		    for (double xi = xrangeStart; xi <= xrangeEnd; xi += xstep) {
		    	for (double yi = yrangeStart; yi <= yrangeEnd; yi += ystep) {
		    		for (double di = 0.0; di < 360; di += degstep) {
		    			tracefilePositions.put("x" + xi + "y" + yi + "d" + di, false);
		    		}
		    	}
		    }
	    }
		// traceFile öffnen
	    try {
	    	String timestamp = new String();
		    System.out.println("Reading the tracefile ...");
	    	BufferedReader in = new BufferedReader(new FileReader(args[0]));
			
	    	int traceCount = 0;
	    		
			// wurde ein index angegeben?
			if (traceIndex > 0) {
				// evtl. Zeilen überspringen
				int j = 0;
				while (traceIndex > 0) {
					String line;
					if ((line = in.readLine()) != null) {
						if (line.startsWith("# timestamp=")) {
							timestamp = line.substring(12);
							traceIndex--;
						}
					} else {
						System.out.println("Error: Dataset index too high!");
						System.exit(-1);
					}
					j++;
				}
				System.out.println("skipped " + (j - 1) + " lines");
			} else if (!useAllTraces) {
				StringBuffer sb = new StringBuffer();
				// prüfen, ob mehrere Traces enthalten sind
				System.out.println("Checking for multiple traces ...");
				String line;
			    while ((line = in.readLine()) != null) {
			    	if (line.startsWith("# timestamp=")) {
			    		traceCount++;
			    		sb.append("   " + (traceCount < 10 ? " " : "") + traceCount + ": " + line.substring(12));
			    		// solange weiterlesen, bis eine Zeile kommt, die kein Kommentar ist
			    		do {
			    			line = in.readLine();
			    		} while ((line != null) && (line.startsWith("#")));
			    		if (line != null) {
			    			int i = line.indexOf("pos=");
			    			int j = line.indexOf("degree=", i);
			    			sb.append(" (" + line.substring(i, line.indexOf(";", j)) + ")");
			    		}
			    		sb.append("\n");
			    	}
			    }
			    if (traceCount == 0) {
			    	System.out.println("Error: Tracefile contains no trace data!");
			    	System.exit(-1);
			    } else if (traceCount > 1) {
			    	System.out.println("Error: Tracefile contains more than one dataset:");
			    	System.out.print(sb.toString());
			    	System.out.println("Please specify one or all of the datasets using the -t option.");
			    	System.exit(-1);
			    } else {
			    	System.out.println("The tracefile contains only one trace.");
			    	in.close();
			    	in = new BufferedReader(new FileReader(args[0]));
			    	sb.delete(0, sb.length());
			    }
			}
			
		    // zeilenweise einlesen
			int pollCount = 0;
			double apCount=0;
			
			traceCount = -1;
			
			String line;
			
			while ((line = in.readLine()) != null) {
				boolean pollHasMultipleEntriesPerAP = false;
				if (line.startsWith("#")) {
					if (line.substring(2).startsWith("timestamp=")) {
						traceCount++;
						if (traceCount > 0) { // dies ist nicht der erste Trace
							Iterator<String> apIterator = apSeenInThisTrace.keySet().iterator();
							while (apIterator.hasNext()) {
								String key = apIterator.next();
								if (!apSeenInThisTrace.get(key)) { // der AP wurde im letzten Trace nicht gefunden
									if (minimumMacSetTraces.contains(key))
										minimumMacSetTraces.remove(key);
								} else {
									// Status zurücksetzen
									apSeenInThisTrace.put(key, false);
								}
							}
						}
						// wenn timestamp hier schon einen Wert enthält,
						// wurde gerade der Beginn des nächsten Daten-
						// satzes erreicht
						if ((!timestamp.equals("")) && (!useAllTraces)) {
							break; // while-Schleife verlassen
						} else
							timestamp = line.substring(12);
					}
					continue;
				}
				// Zeile zerlegen
				double degree = -1.0;
				double mappedDegree = -1.0;
				String[] components = line.split(";");
				for (String s: components) {
					String[] kv = s.split("=", 2);
					if (kv.length != 2)
						throw new IOException("Expected key=value");
					String key = kv[0];
					String value = kv[1];
				    // unwichtige Teile verwerfen
					if ((key.equals("t")) || (key.equals("id")))
						continue;
					// Koordinaten für diese Zeile auslesen
					if ((key.equals("pos"))) {
						String[] coordinates = value.split(",");
						currentX = Double.parseDouble(coordinates[0]);
						currentY = Double.parseDouble(coordinates[1]);
						continue;
					}
					// Richtung für diese Zeile auslesen
					if ((key.equals("degree"))) {
						degree = Double.parseDouble(value);
						mappedDegree = new Double(Math.round(degree / degmap) * degmap);
						if (mappedDegree == 360.0)
							mappedDegree = 0.0;
						continue;
					}
				    if (checkTrainingSet) {
						// Den Punkt currentX/currentY mit Richtung degree als im
						// Tracefile vorhanden markieren
				    	String posString= "x" + currentX + "y" + currentY + "d" + mappedDegree;
						if (!tracefilePositions.containsKey(posString))
							System.out.println("Unexpected position or orientation: " + currentX + "/" + currentY + " at " + mappedDegree + " degrees!");
						else
							tracefilePositions.put(posString, true);
				    }
					// Sendestärke, Frequenz und Modus parsen
					String[] values = value.split(",", 3);
					if (values.length != 3)
						throw new IOException("Expected MAC=rss,frequency,mode");
					int rss = Integer.parseInt(values[0]);
					int mode = Integer.parseInt(values[2]);
					// Ad-Hoc-Knoten überspringen
					if (mode == 1)
						continue;
					// MAC-Adresse schon bekannt?
					AccessPoint ap;
					if (aps.containsKey(key)) {
						// ja: reading hinzufügen
						ap = aps.get(key);
						ap.addReading(rss, degree);
					} else {
					    // nein: AP hinzufügen
						ap = new AccessPoint(key);
						// der neue AP kam bei pollCount Messungen bisher nicht
						// vor, deshalb "nachtragen"
						for (int i = 0; i < pollCount; i++)
							ap.addReading(999, degree);
						ap.addReading(rss, degree);
						aps.put(key, ap);
					}
					if ((!apSeenInThisPoll.containsKey(key)) || (!apSeenInThisPoll.get(key))) {
						// der AP wurde in diesem poll bisher nicht gefunden
						apCount++;
						ap.incPollCount();
					} else {
						// der AP kam in diesem poll bereits vor
						if (!pollHasMultipleEntriesPerAP) {
							pollHasMultipleEntriesPerAP = true;
							pollsWithMultipleEntriesPerAP++;
						}
					}
					apSeenInThisPoll.put(key, true);
					apSeenInThisTrace.put(key, true);
					// Minimales MAC-Set über die polls:
					// Beim ersten Poll alle gefundenen APs zum MAC-Set
					// hinzufügen, bei den nachfolgenden Polls alle APs aus
					// dem MAC-Set löschen, die nicht gefunden wurden.
					if ((pollCount == 0) && (!minimumMacSetPolls.contains(key)))
						minimumMacSetPolls.add(key);
					// Minimales MAC-Set über die traces analog:
					if (traceCount == 0) {
						if (!minimumMacSetTraces.contains(key)) {
							minimumMacSetTraces.add(key);
						}
					}
				}
				pollCount++;
				// APs, die in der zuletzt gelesenen Zeile nicht vorkamen,
				// "nachtragen"
				Iterator<String> apIterator = apSeenInThisPoll.keySet().iterator();
				while (apIterator.hasNext()) {
					String key = apIterator.next();
					if (!apSeenInThisPoll.get(key)) {
						// AP wurde nicht gefunden, also nachtragen
						aps.get(key).addReading(999, degree);
						// falls der AP noch zum minimalen MAC-Set gehört,
						// muss er aus diesem entfernt werden
						if (minimumMacSetPolls.contains(key))
							minimumMacSetPolls.remove(key);
					} else {
						// Status zurücksetzen
						apSeenInThisPoll.put(key, false);
					}
				}
			}
			System.out.println(pollCount + " lines read, " + pollsWithMultipleEntriesPerAP + " (" + (pollsWithMultipleEntriesPerAP * 100 / pollCount) + " %) of them contained multiple entries for at least one AP)");
			// traceFile schliessen
			in.close();
			
			// HashTable in Array umwandeln, da im Anschluss mehrfach durch die APs
			// iteriert werden muss und Iteratoren nicht immer die gleiche Reihenfolge
			// garantieren. Durch die Umwandlung in ein Array muss nur einmal iteriert
			// werden.
			apList = new ArrayList<AccessPoint>();
			Iterator<String> apIterator = aps.keySet().iterator();
			int count = 1;
			int apIndex = 999;
			while (apIterator.hasNext()) {
				AccessPoint ap = aps.get(apIterator.next());
				ap.createHistogram();
				if ((apIndex == 999) || (apIndex == count))
					apList.add(ap);
				count++;
				if (count > apIndex)
					break;
			}

			// Anzahl der AccessPoints
			System.out.println("\n" + apList.size() + " access points found:");
			for (int i = 0; i < apList.size(); i++) {
				AccessPoint ap = apList.get(i);
				System.out.println("   " + ap.getMacAddr() + " (" + ap.getRssCount(0, 102) + " readings in " + ap.getPollCount() + " polls)");
			}
			
			// durchschnittliche AP-Zahl pro poll
			System.out.println("\nAverage number of access points per poll: " + (apCount / pollCount));
			
			// minimales MAC-Set über die Polls (alle MAC-Adressen, die in jedem Poll vorkommen)
			System.out.println("\nMinimum MAC-Set of all polls:");
			if (minimumMacSetPolls.size() == 0) {
				System.out.println("   empty");
			} else {
				for (int i = 0; i < minimumMacSetPolls.size(); i++) {
					System.out.println("   " + minimumMacSetPolls.get(i));
				}
			}
			
			// APs, die nicht zum minimalen MAC-Set über die Polls gehören
			System.out.println("\nAccess Points that are not contained in the minimum MAC-Set of all polls:");
			if (minimumMacSetPolls.size() == apList.size()) {
				System.out.println("   none");
			} else {
				for (int i = 0; i < apList.size(); i++) {
					String key = apList.get(i).getMacAddr();
					if (!minimumMacSetPolls.contains(key))
						System.out.println("   "  + key);
				}
			}
			
			if (useAllTraces) {
				// minimales MAC-Set über die Traces (alle MAC-Adressen, die in jedem Trace vorkommen)
				System.out.println("\nMinimum MAC-Set of all traces:");
				if (minimumMacSetTraces.size() == 0) {
					System.out.println("   empty");
				} else {
					for (int i = 0; i < minimumMacSetTraces.size(); i++) {
						String key = minimumMacSetTraces.get(i);
						AccessPoint ap = aps.get(key);
						System.out.println("   " + key + " was read in " + ap.getPollCount() + " (" + (100 * ap.getPollCount() / pollCount) + "%) of all polls");
					}
				}
				
				// APs, die nicht zum minimalen MAC-Set über die Traces gehören
				System.out.println("\nAccess Points that are not contained in the minimum MAC-Set of all traces:");
				if (minimumMacSetTraces.size() == apList.size()) {
					System.out.println("   none");
				} else {
					for (int i = 0; i < apList.size(); i++) {
						String key = apList.get(i).getMacAddr();
						if (!minimumMacSetTraces.contains(key))
							System.out.println("   " + key);
					}
				}
			}
			
			if (checkTrainingSet) {
				// Prüfen, ob alle für das Trainingset nötigen Positionen und Richtungen
				// im tracefile vorkamen
				int i = 0;
				System.out.println("\nChecking training set completeness:");
			    for (double xi = xrangeStart; xi <= xrangeEnd; xi += xstep) {
			    	for (double yi = yrangeStart; yi <= yrangeEnd; yi += ystep) {
			    		for (double di = 0.0; di < 360; di += degstep) {
			    			String posString = "x" + xi + "y" + yi + "d" + di;
			    			if (!tracefilePositions.get(posString)) {
			    				System.out.println("   No trace found for position " + xi + "/" + yi + " at " + di + " degrees!");
			    				i++;
			    			}
			    		}
			    	}
			    }
			    if (i != 0)
			    	System.out.println("   The training set is NOT complete!");
			    else
			    	System.out.println("   The training set is complete!");
			}
			System.out.println();
	    } catch (FileNotFoundException e) {
	    	System.err.println("Couldn't open tracefile\n" + e.toString());
	    } catch (IOException e) {
			System.err.println("Couldn't read from tracefile\n" + e.toString());
	    } catch (Exception e) {
	    	System.err.println("Unknown error\n");
	    	e.printStackTrace();
	    }
	}
}