/*
 * This file is part of Locana.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.locana.misc;

import java.io.File;
import java.io.FileWriter;

/**
 * This class generates property files to either
 *  - set the grid dimension and start position of the grid (grid) 
 *  or
 *  - the number of access points used (number)
 *
 * @author thomas
 *
 */
public class PropWriter {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		if (args[0].equals("grid")) {
			double gridDimension = Double.parseDouble(args[1]);
			double startPosX = Double.parseDouble(args[2]);
			double startPosY = Double.parseDouble(args[3]);
			double startPosZ = 0.0;
			if (args.length == 5) {
				startPosZ = Double.parseDouble(args[4]);
			}
			String filename = gridDimension + "__" + startPosX + "_" + startPosY + ".prop";
			FileWriter fw = new FileWriter(new File(filename));
			fw.write("PositionFilterGrid.gridDimension=" + gridDimension +"\n");
			fw.write("PositionFilterGrid.startPosition=" + startPosX + " " + startPosY + " " + startPosZ);
			fw.flush();
			System.out.println("File " + filename + " written.");
		}
		if (args[0].equals("number")) {
			int number = Integer.parseInt(args[1]);
			String filename = number + ".prop";
			FileWriter fw = new FileWriter(new File(filename));
			fw.write("MacFilterNOutOfExplizit.n=" + number +"\n");
			fw.flush();
			System.out.println("File " + filename + " written.");
		}
	}

}
