/*
 * This file is part of Locana.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.locana.misc;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;

import org.pi4.locutil2.GeoPosition;
import org.pi4.locutil2.trace.Parser;
import org.pi4.locutil2.trace.TraceEntry;

/**
 * This programm sketches the area of measurements points for a given trace file.
 * It assumes that the trace file contains an offline set trace with regular grid dimension.
 * 
 * @author king
 *
 */
public class SketchOfTrace {
	public static void main(String args[]) {
		if (args.length != 4) {
			usage();
			System.exit(1);
		}
		try {
			int x = Integer.parseInt(args[0]);
			int y = Integer.parseInt(args[1]);
			int xyStepMulti = Integer.parseInt(args[2]);
			
			int[][] sketch = new int[(x * xyStepMulti) + 1][(y * xyStepMulti) + 1];
			
			// read trace file
			Parser parser = new Parser(new File(args[3]));
			ArrayList<TraceEntry> traceEntries = parser.parse();
			Iterator<TraceEntry> iterTraceEntries = traceEntries.iterator();
			GeoPosition lastPos = new GeoPosition();
			while (iterTraceEntries.hasNext()) {
				TraceEntry entry = iterTraceEntries.next();
				GeoPosition pos = entry.getGeoPosition();
				if (!pos.equalsWithoutOrientation(lastPos)) {
					// assume x * xyStepMulti and y * xyStepMulti are integer values
					// create sketch
					sketch[(int) (pos.getX() * xyStepMulti)][(int) (pos.getY() * xyStepMulti)] += 1;
				}
				lastPos = pos;
			}
			
			// print sketch
			for (int j = (y * xyStepMulti); j >= 0; j--) {
				for (int i = 0; i <= (x * xyStepMulti); i++) {
					if (sketch[i][j] == 0) System.out.print(" ");
					else System.out.print(sketch[i][j]);
				}
				System.out.println();
			}
		} catch (Exception e) {
			System.err.println("Exception:");
			System.err.println(e.getLocalizedMessage());
			e.printStackTrace();
			System.out.println();
			usage();
		}
	}
	
	private static void usage() {
		System.out.println("usage: java SketchOfTrace xDimension yDimension xyMultiplicator tracefile");
		System.out.println();
		System.out.println("This programm sketches the area of measurements points for a given trace file.");
		System.out.println("It assumes that the trace file contains an offline set trace with regular grid dimension.");
		System.out.println(" - xDimension: x dimension of data in trace file");
		System.out.println(" - yDimension: y dimension of data in trace file");
		System.out.println(" - xyMultiplicator: multiplicator to get integer values of a grid dimension smaller 1");
		System.out.println();
	}
}
