/*
 * This file is part of Locana.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.locana.misc;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;


public class Trace2Gnuplot {
	private static PrintStream outputFile;
	private static PrintStream outputFile2;
	private static PrintStream outputControlFile;
	private static ArrayList<AccessPoint> apList;
	
	public static void main(String[] args) {
		Hashtable<String, AccessPoint> aps = new Hashtable<String, AccessPoint>();
		
		
		

		
		
		
		
		
		
		
		
		
		int traceIndex = 0;
		int apIndex = 999;
		int dirstep = 5;
		String outputType = "histogram";
		String prefix = new String();
		boolean useAllTraces = false;
		int gaussfunctions = 10;
		
		
		Hashtable<String, Integer> readings;
		
	    if (args.length < 1) {
	        System.out.println();
	        System.out.println("Usage: trace2gnu traceFile [OPTIONS]");
	        System.out.println("Options:");
	        System.out.println("  -p prefix       prefix with which the names of the generated files should start");
	        System.out.println("  -t index        index of the trace dataset in the file or 'all' to combine all the traces");
	        System.out.println("  -a index        index of the access point to process");
	        System.out.println("  -s dirstep      granularity of the direction scale (dir output only)");
	        System.out.println("  -o type         output type (see below)");
	        System.out.println("  -g gaussians    number of gaussian functions used for fitting the histogram (mgaussfit only)");
	        System.out.println();
	        System.out.println("Output types:");
	        System.out.println("  histogram    frequency of received signal strengths");
	        System.out.println("  gauss        unimodal gaussian representation of the histogram");
	        System.out.println("  rss          received signal strengths over time");
	        System.out.println("  acf          auto-correlation-function");
	        System.out.println("  dir          means and std. deviations of the received signals strengths over different directions");
	        System.out.println("  mgaussfit    multi-modal gaussian representation of the histogram (scaled to fit the values)");
	        //System.out.println("  mgaussprob   multi-modal gaussian representation of the histogram (probabilities)");
	        System.out.println();
	        return;
	    }
	    if (args.length>1) { // es wurden Optionen angegeben
	    	for (int i = 0; i < args.length; i++) {
				if (args[i].equalsIgnoreCase("-t")) {
					if ((i + 1) < args.length) { // es gibt noch ein weiteres Argument
						if (args[i + 1].equals("all")) {
							useAllTraces = true;
						} else {
							traceIndex = Integer.parseInt(args[i + 1]);
						}
					}
				} else if (args[i].equalsIgnoreCase("-a")) {
					if ((i + 1) < args.length) { // es gibt noch ein weiteres Argument
						apIndex = Integer.parseInt(args[i + 1]);
					}
				} else if (args[i].equalsIgnoreCase("-o")) {
					if ((i + 1)<args.length) { // es gibt noch ein weiteres Argument
						outputType = args[i + 1];
					}
				} else if (args[i].equalsIgnoreCase("-s")) {
					if ((i + 1)<args.length) { // es gibt noch ein weiteres Argument
						dirstep = Integer.parseInt(args[i + 1]);
					}
				} else if (args[i].equalsIgnoreCase("-p")) {
					if ((i + 1)<args.length) { // es gibt noch ein weiteres Argument
						prefix = args[i + 1];
					}
				} else if (args[i].equalsIgnoreCase("-g")) {
					if ((i + 1)<args.length) { // es gibt noch ein weiteres Argument
						gaussfunctions = Integer.parseInt(args[i + 1]);
					}
				}
	    	}
	    }
		// traceFile öffnen
	    try {
		    BufferedReader in = new BufferedReader(new FileReader(args[0]));
		    String timestamp = new String();
			String line;
			
			// wurde ein index angegeben?
			if (traceIndex > 0) {
				// evtl. Zeilen überspringen
				int j = 0;
				while (traceIndex > 0) {
					if ((line = in.readLine()) != null) {
						if (line.startsWith("# timestamp=")) {
							timestamp = line.substring(12);
							traceIndex--;
						}
					} else {
						System.out.println("Error: Dataset index too high!");
						System.exit(-1);
					}
					j++;
				}
				System.out.println("skipped " + (j - 1) + " lines");
			} else if (!useAllTraces) {
				// prüfen, ob mehrere Traces enthalten sind
				StringBuffer sb = new StringBuffer();
				int traceCount = 0;
			    while ((line = in.readLine()) != null) {
			    	if (line.startsWith("# timestamp=")) {
			    		traceCount++;
			    		sb.append("   " + (traceCount < 10 ? " " : "") + traceCount + ": " + line.substring(12));
			    		// ersten traceEntry suchen
			    		while ((line.startsWith("#")) && (line != null)) {
			    			line = in.readLine();
			    		}
		    			if (line != null) { // traceEntry gefunden
			    			sb.append(", " + line.substring(line.indexOf("pos="), line.indexOf(";", line.indexOf("degree"))));
		    			}
			    		sb.append("\n");
			    	}
			    }
			    if (traceCount==0) {
			    	System.out.println("Error: Tracefile contains no trace data!");
			    	System.exit(-1);
			    } else if (traceCount>1) {
			    	System.out.println("Error: Tracefile contains more than one dataset:");
			    	System.out.print(sb.toString());
			    	System.out.println("Please specify one of the datasets using the -t option.");
			    	System.exit(-1);
			    } else {
			    	in.close();
			    	in = new BufferedReader(new FileReader(args[0]));
			    }
			}
			
		    // zeilenweise einlesen
			int pollCount = 0;
			int usec = -1;
			int minReadings = -1;
			while ((line = in.readLine()) != null) {
				try {
					if (line.startsWith("#")) {
						if (line.substring(2).startsWith("timestamp=")) {
							// wenn timestamp hier schon einen Wert enthält,
							// wurde gerade der Beginn des nächsten Daten-
							// satzes erreicht
							if ((!timestamp.equals("")) && (!useAllTraces)) {
								//System.out.println("start of next dataset reached");
								break; // while-Schleife verlassen
							} else
								timestamp=line.substring(12);
						} else if (line.substring(2).startsWith("usec=")) {
							usec = Integer.parseInt(line.substring(7));
						} else if (line.substring(2).startsWith("minReadings=")) {
							minReadings = Integer.parseInt(line.substring(14));
						}
						continue;
					}
				    // Zeile zerlegen
					readings = new Hashtable<String, Integer>();
					double degree = -1.0;
					String[] components = line.split(";");
					for (String s: components) {
						String[] kv = s.split("=", 2);
						if (kv.length != 2)
							throw new IOException("Expected key=value");
						String key = kv[0];
						String value = kv[1];
					    // unwichtige Teile verwerfen
						if ((key.equals("t"))||(key.equals("id"))||(key.equals("pos")))
							continue;
						// Richtung für diese Zeile auslesen
						if ((key.equals("degree"))) {
							degree = Double.parseDouble(value);
							continue;
						}
					    // Sendestärke parsen
						int rss = Integer.parseInt(value.substring(0, value.indexOf(",")));
						readings.put(key, rss);
					}
					Iterator<String> apIterator = readings.keySet().iterator();
					while (apIterator.hasNext()) {
						String key = apIterator.next();
						// MAC-Adresse schon bekannt?
						if (aps.containsKey(key)) {
							// ja: reading hinzufügen
							aps.get(key).addReading(readings.get(key), degree);
						} else {
						    // nein: AP hinzufügen
							AccessPoint ap = new AccessPoint(key);
							// der neue AP kam bei pollCount Messungen bisher nicht
							// vor, deshalb "nachtragen"
							for (int i = 0; i < pollCount; i++)
								ap.addReading(999, degree);
							ap.addReading(readings.get(key), degree);
							aps.put(key, ap);
						}
					}
					pollCount++;
					// APs, die in der zuletzt gelesenen Zeile nicht vorkamen,
					// "nachtragen"
					apIterator = aps.keySet().iterator();
					while (apIterator.hasNext()) {
						AccessPoint ap = aps.get(apIterator.next());
						if (ap.getRssCount(0, 102) < pollCount)
							ap.addReading(999, degree);
					}
				}
				catch (IOException e) {
					System.err.println("Couldn't read from tracefile\n" + e.toString());
				}
			}
			System.out.println(pollCount + " lines read");
			// traceFile schliessen
			in.close();
			
			// HashTable in Array umwandeln, da im Anschluss mehrfach durch die APs
			// iteriert werden muss und Iteratoren nicht immer die gleiche Reihenfolge
			// garantieren. Durch die Umwandlung in ein Array muss nur einmal iteriert
			// werden.
			apList = new ArrayList<AccessPoint>();
			Iterator<String> apIterator = aps.keySet().iterator();
			int l = 1;
			while (apIterator.hasNext()) {
				AccessPoint ap = aps.get(apIterator.next());
				if ((apIndex == 999) || (apIndex == l))
					apList.add(ap);
				l++;
				if (l > apIndex)
					break;
			}

			// outputFiles öffnen
			// Dateiname mit Pfadangabe?
			String filenamePart1 = new String();
			String filenamePart2 = new String();
			if (args[0].contains("/")) {
				filenamePart1 = args[0].substring(0, args[0].lastIndexOf("/") + 1);
				if (prefix.equals(""))
					filenamePart2 = args[0].substring(args[0].lastIndexOf("/") + 1, args[0].lastIndexOf(".trace"));
				else
					filenamePart2 = prefix;
			} else {
				if (prefix.equals(""))
					filenamePart2 = args[0].substring(0, args[0].lastIndexOf(".trace"));
				else
					filenamePart2 = prefix;
			}
			try {
	    		outputFile = new PrintStream(new FileOutputStream(filenamePart1 + filenamePart2 + ".data"));
	    		outputFile2 = new PrintStream(new FileOutputStream(filenamePart1 + filenamePart2 + ".gaussdata"));
	    		outputControlFile = new PrintStream(new FileOutputStream(filenamePart1 + filenamePart2 + ".gnuplot"));
	    	} catch (IOException e) {
	    		System.err.println("Couldn't open output file\n" + e.toString());
	    		System.exit(1);
	    	}

	    	if ((outputType.equals("histogram")) || (outputType.equals("gauss"))) {
			// Histogramme aufbauen
	    		for (int i = 0; i < apList.size(); i++) {
	    			apList.get(i).createHistogram();
	    		}
	    		// durch die möglichen Sendestärken iterieren
	    		StringBuffer sb = new StringBuffer();
	    		for (int i = -102; i <= 0; i++) {
					sb.append(Integer.toString(i) + "\t");
					for (int j = 0; j < apList.size(); j++) {
						sb.append(Integer.toString(apList.get(j).getRssFrequency(i)) + "\t");
					}
					outputFile.println(sb.toString());
				}
				
				// gnuplot-Steuerdatei erstellen
		    	outputControlFile.println("set style data " + (outputType.equals("gauss") ? "impulses" : "linespoints"));
		    	outputControlFile.println("set xrange [-102:0]");
		    	outputControlFile.println("set title 'histogram, start:" + timestamp + ", timeout=" + usec + ", minReadings=" + minReadings + "'");
		    	outputControlFile.println("set xlabel 'signal strength'");
		    	outputControlFile.println("set ylabel 'readings'");
				// durch die APs iterieren
		    	StringBuffer sb2 = new StringBuffer();
		    	for (int i = 0; i < apList.size(); i++) {
		    		AccessPoint ap = apList.get(i);
					if (i == 0)
						sb2.append("plot '");
					else
						sb2.append(", '");
					sb2.append(filenamePart2);
					sb2.append(".data' using 1:");
					sb2.append(Integer.toString(i + 2));
					sb2.append(" title '");
					sb2.append(ap.getMacAddr());
					sb2.append(" (mean=");
					sb2.append(Double.toString(Math.round(ap.getMean(0, 102) * 100.0) / 100.0));
					sb2.append(", std=");
					sb2.append(Double.toString(Math.round(ap.getStandardDeviation(0, 102) * 100.0) / 100.0));
					sb2.append(")'");
				}
				
				// evtl. Gauss-Glocken einzeichnen
		    	if (outputType.equals("gauss")) {
					for (int i = 0; i < apList.size(); i++) {
						AccessPoint ap = apList.get(i);
						double std = ap.getStandardDeviation(0, 102);
						outputControlFile.println("g" + i + "(x)=" + ap.getRssCount(0, 102) + "*(exp(-0.5*(((x-" + ap.getMean(0, 102) + ")/" + std + ")**2)))/(" + std + "*sqrt(2*pi))");
						sb2.append(", g");
						sb2.append(i);
						sb2.append("(x) with lines linewidth 2");
			   		}
		    	}
		    	outputControlFile.println(sb2.toString());
	    	} else if (outputType.equals("gaussfilter")) {
				// Histogramme aufbauen
	    		for (int i = 0; i < apList.size(); i++) {
	    			AccessPoint ap = apList.get(i);
	    			ap.createHistogram();
	    			ap.applyGaussianFilter();
	    		}
	    		// durch die möglichen Sendestärken iterieren
	    		StringBuffer sb = new StringBuffer();
				for (int i = -102; i <= 0; i++) {
					sb.append(Integer.toString(i) + "\t");
					for (int j = 0; j < apList.size(); j++) {
						sb.append(Double.toString(apList.get(j).getFilteredRssFrequency(i)) + "\t");
					}
					outputFile.println(sb.toString());
				}
				
				// gnuplot-Steuerdatei erstellen
		    	outputControlFile.println("set style data linespoints");
		    	outputControlFile.println("set xrange [-102:0]");
		    	outputControlFile.println("set title 'gaussfilter, start:" + timestamp + ", timeout=" + usec + ", minReadings=" + minReadings + "'");
		    	outputControlFile.println("set xlabel 'signal strength'");
		    	outputControlFile.println("set ylabel 'readings'");
				// durch die APs iterieren
		    	StringBuffer sb2 = new StringBuffer();
		    	for (int i = 0; i < apList.size(); i++) {
		    		AccessPoint ap = apList.get(i);
					if (i == 0)
						sb2.append("plot '");
					else
					sb2.append(", '");
					sb2.append(filenamePart2);
					sb2.append(".data' using 1:");
					sb2.append(Integer.toString(i + 2));
					sb2.append(" title '");
					sb2.append(ap.getMacAddr());
					sb2.append("'");
				}
				
		    	outputControlFile.println(sb2.toString());
			} else if (outputType.equals("acf")) {
				StringBuffer sb = new StringBuffer();
				for (int i = 0; i < 10; i++) {
					sb.append(Integer.toString(i) + " ");
					for (int j = 0; j < apList.size(); j++) {
						AccessPoint ap = apList.get(j);
						sb.append(Double.toString(ap.getVariance(i) / ap.getVariance(0)));
						sb.append(" ");
					}
					outputFile.println(sb.toString());
				}
				// gnuplot-Steuerdatei erstellen
		    	outputControlFile.println("set style data linespoints");
		    	outputControlFile.println("set title 'auto-correlation, start:" + timestamp + ", timeout=" + usec + ", minReadings=" + minReadings + "'");
		    	outputControlFile.println("set xlabel 'lag'");
		    	outputControlFile.println("set ylabel 'ACF'");
				// durch die APs iterieren
		    	StringBuffer sb2 = new StringBuffer();
		    	for (int i = 0; i < apList.size(); i++) {
		    		AccessPoint ap = apList.get(i);
					if (i == 0)
						sb2.append("plot '");
					else
						sb2.append(", '");
					sb2.append(filenamePart2);
					sb2.append(".data' using 1:");
					sb2.append(Integer.toString(i+2));
					sb2.append(" title '");
					sb2.append(ap.getMacAddr());
					sb2.append("'");
				}
				outputControlFile.println(sb2.toString());
			} else if (outputType.equals("rss")) {
				StringBuffer sb = new StringBuffer();
				for (int i = 0; i < pollCount; i++) {
					sb.append(Integer.toString(i));
					sb.append("\t");
					for (int j = 0; j < apList.size(); j++) {
						AccessPoint ap = apList.get(j);
						int rss = ap.getRss(i);
						sb.append((rss == 999 ? "-" : Double.toString(rss)));
						sb.append("\t");
					}
					outputFile.println(sb.toString());
				}
				// gnuplot-Steuerdatei erstellen
		    	outputControlFile.println("set style data points");
		    	outputControlFile.println("set title 'rss, start:" + timestamp + ", timeout=" + usec + ", minReadings=" + minReadings + "'");
		    	outputControlFile.println("set xlabel 'poll'");
		    	outputControlFile.println("set ylabel 'signal strength'");
				// durch die APs iterieren
		    	StringBuffer sb2 = new StringBuffer();
		    	for (int i = 0; i < apList.size(); i++) {
		    		AccessPoint ap = apList.get(i);
		    		if (i == 0)
		    			sb2.append("plot '");
		    		else
		    			sb2.append(", '");
		    		sb2.append(filenamePart2);
		    		sb2.append(".data' using 1:");
		    		sb2.append(Integer.toString(i+2));
		    		sb2.append(" title '");
		    		sb2.append(ap.getMacAddr());
		    		sb2.append("'");
				}
				outputControlFile.println(sb2.toString());
			} else if (outputType.equals("dir")) {
				double mean;
				double std;
				// rss-Durchschnittswerte aufbauen
	    		for (int i = 0; i < apList.size(); i++) {
	    			apList.get(i).createDirDeviations();
	    		}
	    		StringBuffer sb = new StringBuffer();
				for (int i = 0; i < 360; i += dirstep) {
					sb.append(Integer.toString(i));
					sb.append("\t");
					for (int j = 0; j < apList.size(); j++) {
						AccessPoint ap = apList.get(j);
						mean = ap.getDirMean(i);
						std = ap.getDirDeviation(i);
						sb.append(((mean == 999.0) ? "-\t-\t-\t" : Double.toString(mean) + "\t" + Double.toString(mean - std) + "\t" + Double.toString(mean + std) + "\t"));
					}
					outputFile.println(sb.toString());
				}
				// gnuplot-Steuerdatei erstellen
		    	outputControlFile.println("set title 'directions, start:"+timestamp+", timeout="+usec+", minReadings="+minReadings+"'");
		    	outputControlFile.println("set xlabel 'direction'");
		    	outputControlFile.println("set ylabel 'signal strength'");
				// durch die APs iterieren
		    	StringBuffer sb2 = new StringBuffer();
		    	for (int i = 0; i < (apList.size() * 3); i += 3) {
		    		AccessPoint ap = apList.get(i / 3);
					if (i == 0)
						sb2.append("plot '");
					else
						sb2.append(", '");
					sb2.append(filenamePart2);
					sb2.append(".data' using 1:");
					sb2.append(Integer.toString(i + 2));
					sb2.append(" with linespoints lt ");
					sb2.append(((i / 3) + 1));
					sb2.append(" title '");
					sb2.append(ap.getMacAddr());
					sb2.append("', '");
					sb2.append(filenamePart2);
					sb2.append(".data' using 1:");
					sb2.append(Integer.toString(i + 2));
					sb2.append(":");
					sb2.append(Integer.toString(i + 3));
					sb2.append(":");
					sb2.append(Integer.toString(i + 4));
					sb2.append(" with yerrorbars lt ");
					sb2.append(((i / 3) + 1));
					sb2.append(" notitle");
				}
				outputControlFile.println(sb2.toString());
			} else if ((outputType.equals("mgaussfit")) || (outputType.equals("mgaussstep"))) {
				int max, left, right;
				double mean[] = new double[gaussfunctions], std[] = new double[gaussfunctions], scale[] = new double[gaussfunctions];

				// Histogramme erstellen und mit Gaussfilter glätten
				for (int i = 0; i < apList.size(); i++) {
					AccessPoint ap = apList.get(i);
					ap.createHistogram();
	    			ap.applyGaussianFilter();
	    		}

				// gnuplot-Steuerdatei erstellen
		    	outputControlFile.println("set xrange [-102:0]");
		    	outputControlFile.println("set title 'mgauss2, start:" + timestamp + ", timeout=" + usec + ", minReadings=" + minReadings + "'");
		    	outputControlFile.println("set xlabel 'signal strength'");
		    	outputControlFile.println("set ylabel 'readings'");
		    	if (outputType.equals("mgaussfit")) {
			    	outputControlFile.println("set y2label 'probability'");
			    	outputControlFile.println("set y2tics");
		    	} else {
			    	outputControlFile.println("set y2label ''");
			    	outputControlFile.println("set noy2tics");
		    	}

				// durch die APs iterieren
		    	StringBuffer sb = new StringBuffer();
		    	for (int i = 0; i < apList.size(); i++) {
		    		AccessPoint ap = apList.get(i);
					System.out.println("\n" + ap.getMacAddr());
					if (i == 0)
						sb.append("plot '");
					else
						sb.append(", '");
					if (outputType.equals("mgaussfit")) {
						sb.append(filenamePart2);
						sb.append(".data' using 1:");
						sb.append(Integer.toString(i + 2));
						sb.append(" with impulses lt ");
						sb.append((i + 1));
						sb.append(" title '");
						sb.append(ap.getMacAddr());
						sb.append("'");
						sb.append(", '");
						sb.append(filenamePart2);
						sb.append(".gaussdata' using 1:");
						sb.append(Integer.toString(i + 2));
						sb.append(" with lines linewidth 2 lt ");
						sb.append((i + 1));
						sb.append(" notitle axis x1y2");
					} else {
						sb.append(filenamePart2);
						sb.append(".data' using 1:");
						sb.append(Integer.toString(i + 2));
						sb.append(" with linespoints lt ");
						sb.append(i + 1);
						sb.append(" title '");
						sb.append(ap.getMacAddr());
						sb.append("'");
					}
					for (int j = 0; j < gaussfunctions; j++) {
						mean[j] = 0.0;
						std[j] = 0.0;
						scale[j] = 0.0;
					}
	    			for (int j = 0; j < gaussfunctions; j++) {
	    				max = ap.getFilteredMax();
	    				if (max > 0.0) {
		    				System.out.println("max at " + (max - 102));
		    				left = ap.getLeftBoundary(max);
		    				right = ap.getRightBoundary(max);
		    				System.out.println("left = " + (left - 102) + ", right = " + (right - 102));
		    				mean[j] = ap.getMean(left, right);
		    				std[j] = ap.getStandardDeviation(left, right);
		    				scale[j] = ap.getRssCount(left, right);
		    				System.out.println("mean[" + j + "] = " + mean[j] + ", std[" + j + "] = " + std[j] + ", scale[" + j + "] = " + scale[j]);
		    				ap.substractFromHistogramCopy(mean[j], std[j], scale[j]);
		    				ap.applyGaussianFilter();
	    				} else {
	    					System.out.println("No more gaussians needed for this access point.");
	    					break;
	    				}
	    			}
	    			ap.setGaussianApproximation(mean, std, scale);
	    			ap.normalize();
				}
				
		    	outputControlFile.println(sb.toString());

				// Datenfiles erstellen
				// durch die möglichen Sendestärken iterieren
				StringBuffer sb2 = new StringBuffer();
				StringBuffer sb3 = new StringBuffer();
				for (int r = -102; r <= 0; r++) {
					sb3.append(Integer.toString(r));
					sb3.append("\t");
					sb2.append(Integer.toString(r));
					sb2.append("\t");
					for (int i = 0; i < apList.size(); i++) {
						AccessPoint ap = apList.get(i);
						if (outputType.equals("mgaussfit")) {
							sb3.append(Long.toString(Math.round(ap.getRssFrequency(r))));
							sb3.append("\t");
							sb2.append(Double.toString(ap.getGaussianApproximationValue(r)));
							sb2.append("\t");
						} else {
							sb3.append(Long.toString(Math.round(ap.getFilteredRssFrequency(r))));
							sb3.append("\t");
						}
					}
					outputFile.println(sb3.toString());
					outputFile2.println(sb2.toString());
				}
				outputFile2.close();
			}
			
			// Outputfiles schliessen
			outputFile.close();
			outputControlFile.close();
	    } catch (FileNotFoundException e) {
	    	System.err.println("Couldn't open tracefile\n" + e.toString());
	    } catch (Exception e) {
	    	System.err.println("Unknown error\n");
	    	e.printStackTrace();
	    }
	}
}